@@ -45,11 +45,12 @@ __authors__ = ["Jérôme Kieffer"]
 __contact__ = "jerome.kieffer@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "ESRF"
-__date__ = "03/04/2020"
+__date__ = "10/05/2021"
 
 import logging
 logger = logging.getLogger(__name__)
-
+import posixpath
+import numpy
 try:
     import h5py
 except ImportError:
@@ -57,6 +58,11 @@ except ImportError:
 
 from .fabioimage import FabioImage
 from .fabioutils import NotGoodReader
+from .nexus import Nexus
+try:
+    import hdf5plugin
+except ImportError:
+    hdf5plugin = None
 
 
 class EigerImage(FabioImage):
@@ -66,7 +72,7 @@ class EigerImage(FabioImage):
 
     DESCRIPTION = "Eiger data files based on HDF5"
 
-    DEFAULT_EXTENSIONS = ["h5",  "hdf5"]
+    DEFAULT_EXTENSIONS = ["h5", "hdf5"]
 
     def __init__(self, data=None, header=None):
         """
@@ -74,9 +80,12 @@ class EigerImage(FabioImage):
         """
         if not h5py:
             raise RuntimeError("fabio.EigerImage cannot be used without h5py. Please install h5py and restart")
-
+        if data is None:
+            self.dataset = [None]
+        else:
+            self.dataset = [numpy.atleast_3d(data)]
+        self._data = None
         FabioImage.__init__(self, data, header)
-        self.dataset = [data]
         self.h5 = None
 
     def __repr__(self):
@@ -117,7 +126,7 @@ class EigerImage(FabioImage):
             if "data" in entry:
                 data = entry["data"]
                 if isinstance(data, h5py.Group):
-                    "Newer format /entry/data/data_1"
+                    "Newer format /entry/data/data_000001"
                     datasets = [i for i in data.keys() if i.startswith("data")]
                     datasets.sort()
                     try:
@@ -137,18 +146,17 @@ class EigerImage(FabioImage):
                         lstds.append(entry[i])
                 except KeyError:
                     pass
-
-        if not lstds:
+        if lstds:
+            self.dataset = lstds
+        else:
             raise NotGoodReader("HDF5 file does not contain an Eiger-like structure.")
 
-        self.dataset = lstds
-        self._nframes = sum(i.shape[0] for i in lstds)
-
         if frame is not None:
             return self.getframe(int(frame))
         else:
             self.currentframe = 0
-            self.data = self.dataset[0][self.currentframe, :, :]
+
+            self._data = self.dataset[0][self.currentframe,:,:]
             self._shape = None
             return self
 
@@ -157,15 +165,36 @@ class EigerImage(FabioImage):
         try to write image
         :param fname: name of the file
         """
-        if len(self.dataset.shape) == 2:
-            self.dataset.shape = (1,) + self.dataset.shape
-        with h5py.File(fname, mode="w") as h5file:
-            grp = h5file.require_group("entry/data")
-            if len(self.dataset) > 1:
-                for i, ds in enumerate(self.dataset):
-                    grp["data_%06i" % i] = ds
-            else:
-                grp["data"] = self.dataset
+        if hdf5plugin is None:
+            logger.warning("hdf5plugin is needed for bitshuffle-LZ4 compression, falling back on gzip (slower)")
+            compression = {"compression":"gzip",
+                   "compression_opts":1}
+        else:
+            compression = hdf5plugin.Bitshuffle()
+
+        with Nexus(fname, mode="w") as nxs:
+            entry = nxs.new_entry(entry="entry", program_name=None, force_name=True)
+            data_grp = nxs.new_class(entry, "data", "NXdata")
+            entry.attrs["default"] = "data"
+            nxs.h5.attrs["default"] = "entry"
+            for i, ds in enumerate(self.dataset):
+                if ds is None:
+                    # we are in a trouble
+                    data = numpy.atleast_2d(numpy.NaN)
+                elif isinstance(ds, h5py.Dataset):
+                    data = numpy.atleat_2d(ds[()])
+                else:
+                    data = numpy.atleast_2d(ds)
+                if len(data.shape) == 2:
+                    data.shape = (1,) + data.shape
+                chunks = (1,) + data.shape[-2:]
+                if len(self.dataset) > 1:
+                    hds = data_grp.create_dataset(f"data_{i+1:06d}", data=data, chunks=chunks, **compression)
+                elif len(self.dataset) == 1:
+                    hds = data_grp.create_dataset(f"data", data=data, chunks=chunks, **compression)
+                hds.attrs["interpretation"] = "image"
+                if "signal" not in data_grp.attrs:
+                    data_grp.attrs["signal"] = posixpath.split(hds.name)[-1]
 
     def getframe(self, num):
         """ returns the frame numbered 'num' in the stack if applicable"""
@@ -175,14 +204,23 @@ class EigerImage(FabioImage):
                 if isinstance(self.dataset, list):
                     nfr = num
                     for ds in self.dataset:
-                        if nfr < ds.shape[0]:
-                            data = ds[nfr]
-                            break
-                        else:
-                            nfr -= ds.shape[0]
+                        if ds is None or ds.ndim == 2:
+                            if nfr == 0:
+                                data = None
+                            else:
+                                nfr -= 1
+                        elif ds.ndim == 3:
+                            # Stack of images
+                            if (nfr < ds.shape[0]):
+                                data = ds[nfr]
+                                break
+                            else:
+                                nfr -= ds.shape[0]
+
                 else:
                     data = self.dataset[num]
-                new_img = self.__class__(data=data, header=self.header)
+                new_img = self.__class__(data=None, header=self.header)
+                new_img._data = data
                 new_img.dataset = self.dataset
                 new_img.h5 = self.h5
                 new_img._nframes = self.nframes
@@ -204,7 +242,68 @@ class EigerImage(FabioImage):
     def close(self):
         if self.h5 is not None:
             self.h5.close()
-            self.dataset = None
+            self.dataset = []
+
+    @property
+    def nframes(self):
+        """Returns the number of frames contained in this file
+
+        :rtype: int
+        """
+        return sum(i.shape[0] if i.ndim > 2 else 1 for i in self.dataset)
+
+    def get_data(self):
+        if self._data is None:
+            data = None
+            index = self.currentframe
+            if isinstance(self.dataset, list):
+                frame_idx = [len(ds) if (ds is not None and ds.ndim == 3) else 1 for ds in self.dataset]
+                end_ds = numpy.cumsum(frame_idx)
+                for idx, end in enumerate(end_ds):
+                    start = 0 if idx == 0 else end_ds[idx - 1]
+                    if end > index >= start:
+                        ds = self.dataset[idx]
+                        if ds is None or ds.ndim == 2:
+                            data = ds
+                        else:
+                            data = ds[index - start]
+            else:
+                data = self.dataset[index]
+            self._data = data
+        return self._data
+
+    def set_data(self, data, index=None):
+        """Set the data for frame index
+
+        :param data: numpy array
+        :param int index: index of the frame (by default: current one)
+        :raises IndexError: If the frame number is out of the available range.
+        """
+        if index is None:
+            index = self.currentframe
+        if isinstance(self.dataset, list):
+            frame_idx = [len(ds) if (ds is not None and ds.ndim == 3) else 1 for ds in self.dataset]
+            end_ds = numpy.cumsum(frame_idx)
+            nframes = end_ds[-1]
+            if index == nframes:
+                self.dataset.append(data)
+            elif index > nframes:
+                # pad dataset with None ?
+                self.dataset += [None] * (1 + index - len(self.dataset))
+                self.dataset[index] = data
+            else:
+                for idx, end in enumerate(end_ds):
+                    start = 0 if idx == 0 else end_ds[idx - 1]
+                    if end > index >= start:
+                        ds = self.dataset[idx]
+                        if ds is None or ds.ndim == 2:
+                            self.dataset[idx] = data
+                        else:
+                            ds[index - start] = data
+        if index == self.currentframe:
+            self._data = data
+
+    data = property(get_data, set_data)
 
 
 eigerimage = EigerImage
